import shutil
import time
import requests
import docker
import xnat


from .config import (
    SRC_DIR, BUILD_DIR, DOCKER_IMAGE, DOCKER_CONTAINER, XNAT_MNT_DIRS,
    XNAT_PORT, XNAT_ROOT_DIR,
    DOCKER_NETWORK_NAME, XNAT_URI, REGISTRY_PORT, XNAT_USER, XNAT_PASSWORD,
    DOCKER_REGISTRY_IMAGE, DOCKER_REGISTRY_CONTAINER, CONNECTION_ATTEMPTS,
    CONNECTION_ATTEMPT_SLEEP)


def launch_xnat():
    """Starts an XNAT repository within a single Docker container that has
    has the container service plugin configured to access the Docker socket
    to launch sibling containers.

    The archive, prearchive, build, logs and work directories are mounted from
    the host machine at `xnat4tests.config.XNAT_ROOT`
    for convenience and to facilitate methods that mock the
    environment containers run in within the XNAT container service.
    """

    dc = docker.from_env()

    try:
        image = dc.images.get(DOCKER_IMAGE)
    except docker.errors.ImageNotFound:
        shutil.rmtree(BUILD_DIR, ignore_errors=True)
        shutil.copytree(SRC_DIR, BUILD_DIR)
        image, _ = dc.images.build(path=str(BUILD_DIR), tag=DOCKER_IMAGE)
    
    try:
        container = dc.containers.get(DOCKER_CONTAINER)
    except docker.errors.NotFound:
        volumes = {'/var/run/docker.sock': {'bind': '/var/run/docker.sock',
                                            'mode': 'rw'}}
        # Mount in the XNAT root directory for debugging and to allow access
        # from the Docker host when using the container service
        # Clear previous ROOT directories
        shutil.rmtree(XNAT_ROOT_DIR, ignore_errors=True)
        for  dname in XNAT_MNT_DIRS:
            dpath = XNAT_ROOT_DIR / dname
            dpath.mkdir(parents=True)
            volumes[str(dpath)] = {'bind': '/data/xnat/' + dname,
                                   'mode': 'rw'}

        container = dc.containers.run(
            image.tags[0], detach=True, ports={
                '8080/tcp': XNAT_PORT},
            remove=True, name=DOCKER_CONTAINER,
            # Expose the XNAT archive dir outside of the XNAT docker container
            # to simulate what the XNAT container service exposes to running
            # pipelines, and the Docker socket for the container service to
            # to use
            network=docker_network().id,
            volumes=volumes)


    # Need to give time for XNAT to get itself ready after it has
    # started so we try multiple times until giving up trying to connect
    for attempts in range(1, CONNECTION_ATTEMPTS + 1):
        try:
            login = connect()
        except (xnat.exceptions.XNATError, requests.ConnectionError):
            if attempts == CONNECTION_ATTEMPTS:
                raise
            else:
                time.sleep(CONNECTION_ATTEMPT_SLEEP)
        else:
            break

    # Set the path translations to point to the mounted XNAT home directory
    with login:
        login.post('/xapi/docker/server', json={
            # 'id': 2,
            'name': 'Local socket',
            'host': 'unix:///var/run/docker.sock',
            'cert-path': '',
            'swarm-mode': False,
            'path-translation-xnat-prefix': '/data/xnat',
            'path-translation-docker-prefix': str(XNAT_ROOT_DIR),
            'pull-images-on-xnat-init': False,
            'container-user': '',
            'auto-cleanup': True,
            'swarm-constraints': [],
            'ping': True})
    
    return container


def launch_docker_registry():
    xnat_docker_network = docker_network()
    dc = docker.from_env()
    try:
        image = dc.images.get(DOCKER_REGISTRY_IMAGE)
    except docker.errors.ImageNotFound:
        image = dc.images.pull(DOCKER_REGISTRY_IMAGE)

    try:
        container = dc.containers.get(DOCKER_REGISTRY_CONTAINER)
    except docker.errors.NotFound:
        container = dc.containers.run(
            image.tags[0], detach=True,
            ports={'5000/tcp': REGISTRY_PORT},
            network=xnat_docker_network.id,
            remove=True, name=DOCKER_REGISTRY_CONTAINER)
    return container


def stop_xnat():
    launch_xnat().stop()


def stop_docker_registry():
    launch_docker_registry().stop(DOCKER_REGISTRY_CONTAINER)


def docker_network():
    dc = docker.from_env()
    try:
        network = dc.networks.get(DOCKER_NETWORK_NAME)
    except docker.errors.NotFound:
        network = dc.networks.create(DOCKER_NETWORK_NAME)
    return network


def connect():
    return xnat.connect(XNAT_URI, user=XNAT_USER, password=XNAT_PASSWORD)



from pathlib import Path

SRC_DIR = Path(__file__).parent / 'docker-src'
ROOT_DIR = Path.home() / '.xnat4tests'
ROOT_DIR.mkdir(exist_ok=True)
BUILD_DIR = ROOT_DIR / 'build'
XNAT_ROOT_DIR = ROOT_DIR / 'xnat_root'
XNAT_MNT_DIRS = [
    'home/logs', 'home/work', 'build', 'archive', 'prearchive']
DOCKER_IMAGE = 'xnat4tests'
DOCKER_CONTAINER = 'xnat4tests'
DOCKER_HOST = 'localhost'
XNAT_PORT = '8080'  # This shouldn't be changed as it needs to be the same as the internal for the container service to work
DOCKER_REGISTRY_IMAGE = 'registry'
DOCKER_REGISTRY_CONTAINER = 'xnat4tests-docker-registry'
DOCKER_NETWORK_NAME = 'xnat4tests'
REGISTRY_PORT = '80'  # Must be 80 to avoid bug in XNAT CS config
XNAT_URI = f'http://{DOCKER_HOST}:{XNAT_PORT}'
REGISTRY_URI = f'{DOCKER_HOST}:{REGISTRY_PORT}'
XNAT_USER = 'admin'
XNAT_PASSWORD = 'admin'
CONNECTION_ATTEMPTS = 20
CONNECTION_ATTEMPT_SLEEP = 5



import versioneer
from setuptools import setup, find_packages


setup(
    name='xnat4tests',
    version=versioneer.get_version(),
    author='Thomas G. Close',
    author_email='tom.g.close@gmail.com',
    packages=find_packages(),
    url='https://github.com/australian-imaging-service/xnat4tests',
    license='CC0',
    description=('Creates basic XNAT instance for API tests'),
    long_description=open('README.rst').read(),
    install_requires=[
        'docker>=5.0.2',
        'xnat>=0.3.17',
        'requests>=2.10.0'],
    extras_require={
        'test': [
            'pytest>=5.4.3']},
    include_package_data=True,
    cmdclass=versioneer.get_cmdclass(),
    classifiers=(
        ["Development Status :: 4 - Beta",
         "Intended Audience :: Healthcare Industry",
         "Intended Audience :: Science/Research",
         "License :: OSI Approved :: Apache Software License",
         "Natural Language :: English",
         "Topic :: Scientific/Engineering :: Bio-Informatics",
         "Topic :: Scientific/Engineering :: Medical Science Apps."]
        + ["Programming Language :: Python :: " + str(v)
           for v in ('3.5', '3.6', '3.7', '3.8', '3.9')]),
    keywords='repository analysis neuroimaging workflows pipelines')
